package com.webchat.service.queue;

import com.webchat.common.enums.ChatMessageTypeEnum;
import com.webchat.common.enums.RedisKeyEnum;
import com.webchat.common.util.JsonUtil;
import com.webchat.controller.client.ChatWebSocket;
import com.webchat.domain.vo.request.mess.ChatMessageRequestVO;
import com.webchat.domain.vo.response.UserBaseResponseInfoVO;
import com.webchat.domain.vo.response.mess.ChatMessageResponseVO;
import com.webchat.domain.vo.response.mess.PublicAccountArticleMessageVO;
import com.webchat.domain.vo.response.publicaccount.ArticleBaseResponseVO;
import com.webchat.repository.dao.IArticleDAO;
import com.webchat.service.ChatMessService;
import com.webchat.service.FriendService;
import com.webchat.service.UserService;
import com.webchat.service.publicaccount.ArticleService;
import com.webchat.service.queue.dto.ArticleDelayConsumeMessageDTO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import javax.websocket.Session;
import java.util.Set;

/**
 * @author 77
 * @date 2024/11/13 00:01
 */
@Component
public class ArticlePublishQueue extends AbstractRedisQueue<ArticleDelayConsumeMessageDTO> {

    @Autowired
    private FriendService friendService;
    @Autowired
    private ArticleService articleService;
    @Autowired
    private UserService userService;
    @Autowired
    private ChatMessService chatMessService;
    @Autowired
    private IArticleDAO articleDAO;

    @PostConstruct
    public void initBean() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                ArticlePublishQueue.super.schedule();
            }
        }).start();
    }

    @Override
    protected ArticleDelayConsumeMessageDTO convert(String s) {
        if (StringUtils.isBlank(s)) {
            return null;
        }
        return JsonUtil.fromJson(s, ArticleDelayConsumeMessageDTO.class);
    }

    @Override
    protected String getQueueName() {
        return RedisKeyEnum.QUEUE_ARTICLE_PUBLISH_MESSAGE.getKey();
    }

    @Override
    protected void receive(ArticleDelayConsumeMessageDTO data) {
        if (data == null) {
            return;
        }
        // 1. 拿公众号
        String publicAccount = data.getPublicAccount();
        UserBaseResponseInfoVO publicAccountUser = userService.getUserInfoByUserId(publicAccount);
        // 2. 获取文章信息
        Long articleId = data.getArticleId();
        ArticleBaseResponseVO articleBaseResponse = articleService.getArticleDetailFromCache(articleId);
        // 3. 走redis拿公众号下所有订阅用户
        Set<String> subscribeUsers = friendService.listSubscribeFriendIds(publicAccount);
        if (CollectionUtils.isEmpty(subscribeUsers)) {
            return;
        }
        // 4. 处理在线场景推文
        for (String userId : subscribeUsers) {
            // 4.1 获取公众号所有订阅且在线用户（需要实时推文）
            Session session = ChatWebSocket.clients.get(userId).getSession();
            if (session == null || !session.isOpen()) {
                continue;
            }
            // 4.2 构造文章卡片
            PublicAccountArticleMessageVO publicAccountArticleMessageDTO = new PublicAccountArticleMessageVO();
            publicAccountArticleMessageDTO.setArticleId(articleBaseResponse.getId());
            publicAccountArticleMessageDTO.setCover(articleBaseResponse.getCover());
            publicAccountArticleMessageDTO.setDescription(articleBaseResponse.getDescription());
            publicAccountArticleMessageDTO.setTitle(articleBaseResponse.getTitle());
            publicAccountArticleMessageDTO.setTime(System.currentTimeMillis());

            ChatMessageResponseVO publicAccountArticleMessage = new ChatMessageResponseVO();
            publicAccountArticleMessage.setType(ChatMessageTypeEnum.PUBLIC_ACCOUNT_ARTICLE.getType());
            publicAccountArticleMessage.setSenderId(publicAccount);
            publicAccountArticleMessage.setSender(publicAccountUser);
            publicAccountArticleMessage.setReceiverId(userId);
            publicAccountArticleMessage.setMessage(JsonUtil.toJsonString(publicAccountArticleMessageDTO));

            // 3.3 实时推文
            session.getAsyncRemote().sendText(JsonUtil.toJsonString(publicAccountArticleMessage));
        }
        // 5. 处理离线场景推文
        this.doExecWriteData(articleId, publicAccount, subscribeUsers);
    }

    public void doExecWriteData(Long articleId, String publicAccount, Set<String> receiverIds) {
        // 更新文章推送状态
        articleDAO.updatePushedStatus(articleId);
        // 推文数据写库
        for (String userId : receiverIds) {
            ChatMessageRequestVO messVo = new ChatMessageRequestVO();
            messVo.setType(ChatMessageTypeEnum.PUBLIC_ACCOUNT_ARTICLE.getType());
            messVo.setMessage(String.valueOf(articleId));
            messVo.setSenderId(publicAccount);
            messVo.setReceiverId(userId);
            chatMessService.sendMess(messVo);
        }
    }

    @Override
    protected void error(ArticleDelayConsumeMessageDTO data, Exception ex) {

    }
}
